/*******************************************************************************
 * libproxy - A library for proxy configuration
 * Copyright (C) 2008-2010 Dominique Leuenberger <dominique@leuenberger.net>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 ******************************************************************************/

using System;
using System.Reflection;
using System.Runtime.CompilerServices;
using System.Runtime.InteropServices;

[assembly:AssemblyVersion("0.4.6")]
[assembly:AssemblyTitle ("LibProxy#")]
[assembly:AssemblyDescription ("A library that provides automatic proxy configuration management.")]
[assembly:AssemblyCopyright ("Copyright (c) 2006-2009 Nathaniel McCallum")]
[assembly:AssemblyCompany ("")]
[assembly:AssemblyDelaySign(false)]
[assembly:AssemblyKeyFile("libproxy.snk")]
[assembly:CLSCompliant(false)]

